# Unrestricted Model (Table 2)

#########################################
# log likelihood
llik.multi.risk.rho0 <- function(par, y, x){
# the number of parameters
k <- ncol(x)

# extract parameters
beta0 <- par[1:k]
beta1 <- par[(k+1):(2*k)]
beta2 <- par[((2*k)+1):(3*k)]
beta3 <- par[((3*k)+1):(4*k)]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x%*%beta0
h1 <- x%*%beta1
h2 <- x%*%beta2
h3 <- x%*%beta3
# correlated risks
p0 <- exp(h0)/(1+exp(h0)+exp(h1))
p1 <- exp(h1)/(1+exp(h0)+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

# nested log likelihood
llik <- sum(log(p0[y==0])) +
   sum(log(p1[y==1])) +sum(log(1-(p0+p1)[y>1])) +
   sum(log(p2[y==2])) +sum(log(1-p2[y>2])) +
   sum(log(p3[y==3])) +sum(log(1-p3[y>3])) 
return(llik)
}

#########################################
# probabilities of every risk
prob.multi.risk.rho0 <- function(par, y, x){
# the number of parameters
k <- ncol(x)

# extract parameters
beta0 <- par[1:k]
beta1 <- par[(k+1):(2*k)]
beta2 <- par[((2*k)+1):(3*k)]
beta3 <- par[((3*k)+1):(4*k)]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x%*%beta0
h1 <- x%*%beta1
h2 <- x%*%beta2
h3 <- x%*%beta3
# correlated risks
p0 <- exp(h0)/(1+exp(h0)+exp(h1))
p1 <- exp(h1)/(1+exp(h0)+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

return(cbind(p0,p1,p2,p3,(p2*(1-p0-p1)),(p3*(1-p2)*(1-p0-p1))))
}


#########################################
# main model
multi.risk.box.rho0 <- function(formula, ...){
# extract variables
model <- model.frame(formula)
 Y <- as.matrix(model.extract(model, "response"))
 # Y=4(=win:cencored),3(=general),2(=primary),0(=retire),1(=ambition)
X <- as.matrix(model.matrix(formula))
k <- ncol(X)

## optimization 
# starting values
start0 <- start1 <- start2 <- start3 <- rep(0,k)
start0 <- glm(as.integer(Y==0) ~ X[,-1], family=binomial)$coefficients    
start1 <- glm(as.integer(Y==1) ~ X[,-1], family=binomial)$coefficients  
start2 <- glm(as.integer(Y==2) ~ X[,-1], family=binomial)$coefficients
start3 <- glm(as.integer(Y==3) ~ X[,-1], family=binomial)$coefficients
start.all  <- c(start0,start1,start2,start3)
# optimization  
res <- optim(start.all, llik.multi.risk.rho0, method = "BFGS", hessian = TRUE, control = list(fnscale = -1),
             y = Y, x=X)
res$X0 <- X
res$X1 <- X
res$X2 <- X
res$X3 <- X
res$Y <- Y
res$prob <- prob.multi.risk.rho0(res$par, Y, X)
names(res$par) <- c(colnames(X),colnames(X),colnames(X),colnames(X))
return(res)
}

#########################################
# robust standard error

robust.rho0 <- function(hessian, cluster, par, y, x0, x1, x2, x3){
n <- length(y)
par.num <- length(par)

# the number of parameters
k0 <- ncol(x0);k1 <- ncol(x1);k2 <- ncol(x2);k3 <- ncol(x3)  
k <- k0+k1+k2+k3  

# extract parameters
beta0 <- par[1:k0]
beta1 <- par[(k0+1):(k0+k1)]
beta2 <- par[(k0+k1+1):(k0+k1+k2)]
beta3 <- par[(k0+k1+k2+1):k]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x0%*%beta0
h1 <- x1%*%beta1
h2 <- x2%*%beta2
h3 <- x3%*%beta3

# correlated risks
p0 <- exp(h0)/(1+exp(h0)+exp(h1))
p1 <- exp(h1)/(1+exp(h0)+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

# \partial exp(g(x_t)+h_0(t)) / \partial \theta |\theta=\theta_MLE
dxb.db0 <- cbind(x0,            matrix(0,n,k1),matrix(0,n,k2),matrix(0,n,k3))
dxb.db1 <- cbind(matrix(0,n,k0),x1,            matrix(0,n,k2),matrix(0,n,k3))
dxb.db2 <- cbind(matrix(0,n,k0),matrix(0,n,k1),x2,            matrix(0,n,k3))
dxb.db3 <- cbind(matrix(0,n,k0),matrix(0,n,k1),matrix(0,n,k2),            x3)

# u: contirbution of L_i = \partial log h_r / \partial \theta |\theta=\theta_MLE
u <- matrix(NA,n,par.num)
u[y==0] <- ((as.numeric(1-p0)*dxb.db0)-(as.numeric(p1)*dxb.db1))[y==0]
u[y==1] <- ((-as.numeric(p0)*dxb.db0)+(as.numeric(1-p1)*dxb.db1))[y==1]
u[y==2] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(as.numeric(1-p2)*dxb.db2))[y==2] 
u[y==3] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(-as.numeric(p2)*dxb.db2)+(as.numeric(1-p3)*dxb.db3))[y==3]
u[y==4] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(-as.numeric(p2)*dxb.db2)+(-as.numeric(p3)*dxb.db3))[y==4]

cid <- unique(cluster)
nc<- length(cid)
u.sq <- matrix(0,par.num,par.num)
u.dummy <- matrix(0,nc,par.num)
u <- rbind(u,u.dummy)
cluster <- c(cluster,cid)
for(c in 1:nc){
# summing scores within clusters
uc <- apply(u[cluster==cid[c],],2,sum)
# summing scores across clusters
u.sq <- u.sq + (uc%*%t(uc))
}
# variance covariance matrix
cov <- -solve(hessian)
# robust variance covariance matrix
robust.cov <- cov%*%u.sq%*%cov
return(robust.cov)
}
