# Simple SDCR (Table 2)

#########################################
# log likelihood
llik.multi.risk.rho1 <- function(par, y, x0, x1, x2, x3){
# the number of parameters
k0 <- ncol(x0);k1 <- ncol(x1);k2 <- ncol(x2);k3 <- ncol(x3)  
k <- k0+k1+k2+k3

# extract parameters
beta0 <- par[1:k0]
beta1 <- par[(k0+1):(k0+k1)]
beta2 <- par[(k0+k1+1):(k0+k1+k2)]
beta3 <- par[(k0+k1+k2+1):k]
sigma30 <- par[k+1]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x0%*%beta0
h1 <- x1%*%beta1
h2 <- x2%*%beta2
h3 <- x3%*%beta3
# base hazard 
xb3 <- h3-beta3[1]-(beta3[2]*x3[,2])
# correlated risks
p0 <- exp(h0+(sigma30*xb3))/(1+exp(h0+(sigma30*xb3))+exp(h1))
p1 <- exp(h1)/(1+exp(h0+(sigma30*xb3))+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

# nested log likelihood
llik <- (as.integer(y==0)*log(p0) + as.integer(y==1)*log(p1) +
  as.integer(y==2)*(log(p2)+log(1-(p0+p1))) +
  as.integer(y==3)*(log(p3)+log(1-(p0+p1))+log(1-p2)) + 
  as.integer(y==4)*(log(1-p3)+log(1-(p0+p1))+log(1-p2)))
return(sum(llik))
}

#########################################
# probabilities of every risk
prob.multi.risk.rho1 <- function(par, y, x0, x1, x2, x3){
# the number of parameters
k0 <- ncol(x0);k1 <- ncol(x1);k2 <- ncol(x2);k3 <- ncol(x3)  
k <- k0+k1+k2+k3

# extract parameters
beta0 <- par[1:k0]
beta1 <- par[(k0+1):(k0+k1)]
beta2 <- par[(k0+k1+1):(k0+k1+k2)]
beta3 <- par[(k0+k1+k2+1):k]
sigma30 <- par[k+1]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x0%*%beta0
h1 <- x1%*%beta1
h2 <- x2%*%beta2
h3 <- x3%*%beta3
# base hazard 
xb3 <- h3-beta3[1]-(beta3[2]*x3[,2])
# correlated risks
p0 <- exp(h0+(sigma30*xb3))/(1+exp(h0+(sigma30*xb3))+exp(h1))
p1 <- exp(h1)/(1+exp(h0+(sigma30*xb3))+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

return(cbind(p0,p1,p2,p3,(p2*(1-p0-p1)),(p3*(1-p2)*(1-p0-p1))))
}


#########################################
# main model
multi.risk.box.rho1 <- function(formula0,formula1, formula2, formula3, ...){
# extract variables
model <- model.frame(formula0)
 Y <- as.matrix(model.extract(model, "response"))
 # Y=4(=win:cencored),3(=general),2(=primary),0(=retire),1(=ambition)
X0 <- as.matrix(model.matrix(formula0))
X1 <- as.matrix(model.matrix(formula1))
X2 <- as.matrix(model.matrix(formula2))
X3 <- as.matrix(model.matrix(formula3))
k0 <- ncol(X0);k1 <- ncol(X1);k2 <- ncol(X2);k3 <- ncol(X3)
  
## optimization 
# starting values
start0 <- glm(as.integer(Y==0) ~ X0[,-1], family=binomial)$coefficients    
start1 <- glm(as.integer(Y==1) ~ X1[,-1], family=binomial)$coefficients  
start2 <- glm(as.integer(Y==2) ~ X2[,-1], family=binomial)$coefficients
start3 <- glm(as.integer(Y==3) ~ X3[,-1], family=binomial)$coefficients
start.all  <- c(start0,start1,start2,start3,0)
# optimization  
res <- optim(start.all, llik.multi.risk.rho1, method = "BFGS", hessian = TRUE, control = list(fnscale = -1),
             y = Y, x0 = X0, x1 = X1, x2 = X2, x3 = X3)

names(res$par) <- c(colnames(X0),colnames(X1),colnames(X2),colnames(X3),
                    "sigma30")
res$X0 <- X0
res$X1 <- X1
res$X2 <- X2
res$X3 <- X3
res$Y <- Y
res$prob <- prob.multi.risk.rho1(res$par, Y, X0, X1, X2, X3)
return(res)
}

#########################################
# robust standard error

robust.rho1 <- function(hessian, cluster, par, y, x0, x1, x2, x3){
n <- length(y)
par.num <- length(par)
# the number of parameters
k0 <- ncol(x0);k1 <- ncol(x1);k2 <- ncol(x2);k3 <- ncol(x3)  
k <- k0+k1+k2+k3  
# extract parameters
beta0 <- par[1:k0]
beta1 <- par[(k0+1):(k0+k1)]
beta2 <- par[(k0+k1+1):(k0+k1+k2)]
beta3 <- par[(k0+k1+k2+1):k]
sigma30 <- par[k+1]

## probabilities: deterministic nested dependent risks
# linear predictors of hazard by risk
h0 <- x0%*%beta0
h1 <- x1%*%beta1
h2 <- x2%*%beta2
h3 <- x3%*%beta3
# base hazard 
xb3 <- h3-beta3[1]-(beta3[2]*x3[,2])
# correlated risks
p0 <- exp(h0+(sigma30*xb3))/(1+exp(h0+(sigma30*xb3))+exp(h1))
p1 <- exp(h1)/(1+exp(h0+(sigma30*xb3))+exp(h1))
p2 <- 1/(1+exp(-h2))
p3 <- 1/(1+exp(-h3))

# \partial exp(g(x_t)+h_0(t)) / \partial \theta |\theta=\theta_MLE
dxb.db0 <- cbind(x0,            matrix(0,n,k1),matrix(0,n,k2),sigma30*x3,    xb3)
dxb.db1 <- cbind(matrix(0,n,k0),x1,            matrix(0,n,k2),matrix(0,n,k3),c(rep(0,n)))
dxb.db2 <- cbind(matrix(0,n,k0),matrix(0,n,k1),x2,            matrix(0,n,k3),c(rep(0,n)))
dxb.db3 <- cbind(matrix(0,n,k0),matrix(0,n,k1),matrix(0,n,k2),x3,            c(rep(0,n)))

# u: contirbution of L_i = \partial log h_r / \partial \theta |\theta=\theta_MLE
u <- matrix(NA,n,par.num)
u[y==0] <- ((as.numeric(1-p0)*dxb.db0)-(as.numeric(p1)*dxb.db1))[y==0]
u[y==1] <- ((-as.numeric(p0)*dxb.db0)+(as.numeric(1-p1)*dxb.db1))[y==1]
u[y==2] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(as.numeric(1-p2)*dxb.db2))[y==2] 
u[y==3] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(-as.numeric(p2)*dxb.db2)+(as.numeric(1-p3)*dxb.db3))[y==3]
u[y==4] <- ((-as.numeric(p0)*dxb.db0)-(as.numeric(p1)*dxb.db1)+(-as.numeric(p2)*dxb.db2)+(-as.numeric(p3)*dxb.db3))[y==4]

cid <- unique(cluster)
nc<- length(cid)
u.sq <- matrix(0,par.num,par.num)
u.dummy <- matrix(0,nc,par.num)
u <- rbind(u,u.dummy)
cluster <- c(cluster,cid)
for(c in 1:nc){
# summing scores within clusters
uc <- apply(u[cluster==cid[c],],2,sum)
# summing scores across clusters
u.sq <- u.sq + (uc%*%t(uc))
}
# variance covariance matrix
cov <- -solve(hessian)
# robust variance covariance matrix
robust.cov <- cov%*%u.sq%*%cov
return(robust.cov)
}

