##########################################
# estimator codes and libraries are loaded

# SDCR codes
source("rho4.0.1-0.r") # Full SDCR
source("rho1.0.1-0.r") # Simple SDCR
source("rho0.0.1-0.r") # Unrestricted Model
source("rhoall.0.1-0.r") # common auxiliary file for all SDCR models

# Robust standard error for multinomial logit (general use)
source("multinom.robust.sde.0.1-0.r")

# for multinomial logit
library(nnet)
# for reading STATA files
library(foreign) 

##########################################
# The original data is loaded

# You should download the file "career.dta" at
# http://psfaculty.ucdavis.edu/bsjjones/career.dta
Jones.data <- read.dta("career.dta")
id <- Jones.data$memberid*100 + (Jones.data$class-1900) + Jones.data$duration/100
Jones.data <- Jones.data[order(id),]
# the variable .d is replaced by an older version of the original data
load("d.0.1-0.Rdata")
Jones.data$.d <- d
Jones.data <- Jones.data[is.na(Jones.data$.d)==0,]
# missing value of the variable event is corrected
Jones.data[is.na(Jones.data$event)==1,c("event")] <- 0
attach(Jones.data)

# making variables
exittype <- 4 - event
exittype[event==3] <- 0
exittype[event==4] <- 1
log.duration <- log(duration)

###################################################
# Replication of the original analysis 
# by Box-Steffensmeier and Jones (2004, Table 10.6)

# Table 1, Upper Panel
source("replication.of.original.0.1-0.r")
table.BJ.star

############
# Reanalysis

# Table 1, Lower Panel
# SDCR (The lower panel of Table 1 and Full SDCR in Table 2)
# It takes my computer 2 minutes 16 seconds
source("table1.lower.panel.0.1-0.r")
table.rho4.star

# Table 2
# Model fit
# It takes my computer 7 minutes
source("table2.0.1-0.r")
fit

# general hazards of retirees and runners (p. 24)
general.hazard.retirees <- table(exittype)[4]/sum(table(exittype)[4:5]) 
general.hazard.retirees 
general.hazard.runners <- mean(out.rho4$prob[exittype==0,4]) 
general.hazard.runners
# how many times?
general.hazard.runners/general.hazard.retirees 

# Figure 4
source("derivative.0.1-0.r")

# check if the real data and the Monte Carlo simulation are comparable (footnote 13)
# alpha_r + [tau_r log(t)] + [g_r(x_r)]
g0 <- out.rho4$X0%*%out.rho4$par[1:5]
g1 <- out.rho4$X1%*%out.rho4$par[6:12]
g2 <- out.rho4$X2%*%out.rho4$par[13:17]
g3 <- out.rho4$X3%*%out.rho4$par[18:27]
# their empirical mean
round(mean(g0),1)
round(mean(g1),1)
round(mean(g2),1)
round(mean(g3),1)
# x_r beta_r
v0 <- out.rho4$X0[,-c(1:2)]%*%out.rho4$par[3:5]
v1 <- out.rho4$X1[,-c(1:2)]%*%out.rho4$par[8:12]
v2 <- out.rho4$X2[,-c(1:2)]%*%out.rho4$par[15:17]
v3 <- out.rho4$X3[,-c(1:2)]%*%out.rho4$par[20:27]
# their variance
round(var(v0),2)
round(var(v1),2)
round(var(v2),2)
round(var(v3),2)
# variance of [tau_{1(b)} log(t)] + [g_{1(b)}(x_{1(b)})]
round(var(g0),2)
# variance of log(t)
round(var(log.duration),1)

# save all the objects
# save.image("results.0.1-0.Rdata")


